function p=acfold_init(p,nx,varargin) 
% initialization routine for "acfold" demo
% varargin only used to demonstrate symmetric poimesh
% see manual or routine "stanparam" for explanations of settings
%
% standard settings, window arrangement, file names: 
p=stanparam(p); dir=sprintf('%s',inputname(1)); p=setfn(p,dir); screenlayout(p);
if nargin==3 p.mesh.sympoi=varargin{1}; end 
p.usrlam=[3.5 4]; % desired values of the parameter
% PDE standard implementation
p.nc.neq=1;            % set number of equations for pde
p.fuha.G=@acfold_G;         % set functions G in pde
p.fuha.Gjac=@acfold_Gjac;   % set jacobian of G (not required if p.jsw=3)
p.fuha.spjac=@acfold_spjac; % additional jacobian for fold/branch point continuation

% PDE simple implementation
p.sw.sfem=1; p.eqn.c=1; p.eqn.b=0; p.eqn.a=0;
p.fuha.sG=@acfold_sG; p.fuha.sGjac=@acfold_sGjac; 

% domain and mesh
lx=1; ly=0.9; [p.mesh.geo, bc]=recdbc1(lx,ly,1e3);
p.fuha.bc=@(p,u) bc; p.fuha.bcjac=@(p,u) bc;
ny=nx; p=stanmesh(p,nx,ny); p=setbmesh(p); 

% continuation settings
p.nc.ilam=1;       % set primary parameter index (here no other active parameters)
p.sol.ds=0.5; p.nc.nsteps=30; p.nc.lammax=4.5;
p.nc.amod=20; p.nc.lamdtol=0.8; p.sol.xi=1/p.np; 

% initial point and parameters
p.u=0*ones(p.nu,1); 
par(1)=1;    % linear cofficient of f
par(2)=0.25; % diffusion coefficient
par(3)=1;    % nonlinear coefficient of f
p.u = [p.u; par'];
if p.sw.sfem==1; p=setfemops(p); end % generate required FEM matrices for settings